@@ -11,6 +11,128 @@ TESTS_HOME = os.path.abspath(os.path.dirname(__file__))
 TESTS_DATA = os.path.join(TESTS_HOME, "testdata")
 
 
+class TestSubsetTime:
+    nc_poslons = os.path.join(
+        TESTS_DATA, "cmip3", "tas.sresb1.giss_model_e_r.run1.atm.da.nc"
+    )
+
+    def test_simple(self):
+        da = xr.open_dataset(self.nc_poslons).tas
+        yr_st = "2050"
+        yr_ed = "2059"
+
+        out = subset.subset_time(da, start_date=yr_st, end_date=yr_ed)
+        out1 = subset.subset_time(da, start_date=f"{yr_st}-01", end_date=f"{yr_ed}-12")
+        out2 = subset.subset_time(
+            da, start_date=f"{yr_st}-01-01", end_date=f"{yr_ed}-12-31"
+        )
+        np.testing.assert_array_equal(out, out1)
+        np.testing.assert_array_equal(out, out2)
+        np.testing.assert_array_equal(len(np.unique(out.time.dt.year)), 10)
+        np.testing.assert_array_equal(out.time.dt.year.max(), int(yr_ed))
+        np.testing.assert_array_equal(out.time.dt.year.min(), int(yr_st))
+
+    def test_time_dates_outofbounds(self):
+        da = xr.open_dataset(self.nc_poslons).tas
+        yr_st = "1776"
+        yr_ed = "2077"
+
+        with pytest.warns(None) as record:
+            out = subset.subset_time(
+                da, start_date=f"{yr_st}-01", end_date=f"{yr_ed}-01"
+            )
+        np.testing.assert_array_equal(out.time.dt.year.min(), da.time.dt.year.min())
+        np.testing.assert_array_equal(out.time.dt.year.max(), da.time.dt.year.max())
+
+        assert (
+            '"start_date" not found within input date time range. Defaulting to minimum time step in xarray object.'
+            in [str(q.message) for q in record]
+        )
+        assert (
+            '"end_date" not found within input date time range. Defaulting to maximum time step in xarray object.'
+            in [str(q.message) for q in record]
+        )
+
+    def test_warnings(self):
+        da = xr.open_dataset(self.nc_poslons).tas
+
+        with pytest.raises(ValueError) as record:
+            subset.subset_time(da, start_date="2059", end_date="2050")
+
+        with pytest.raises(TypeError):
+            subset.subset_time(da, start_yr=2050, end_yr=2059)
+
+        with pytest.warns(None) as record:
+            subset.subset_time(
+                da, start_date=2050, end_date=2055,
+            )
+        assert (
+            'start_date and end_date require dates in (type: str) using formats of "%Y", "%Y-%m" or "%Y-%m-%d".'
+            in [str(q.message) for q in record]
+        )
+
+    def test_time_start_only(self):
+        da = xr.open_dataset(self.nc_poslons).tas
+        yr_st = "2050"
+
+        # start date only
+        with pytest.warns(None):
+            out = subset.subset_time(da, start_date=f"{yr_st}-01")
+        np.testing.assert_array_equal(out.time.dt.year.min(), int(yr_st))
+        np.testing.assert_array_equal(out.time.dt.year.max(), da.time.dt.year.max())
+
+        with pytest.warns(None):
+            out = subset.subset_time(da, start_date=f"{yr_st}-07")
+        np.testing.assert_array_equal(out.time.dt.year.min(), int(yr_st))
+        np.testing.assert_array_equal(out.time.min().dt.month, 7)
+        np.testing.assert_array_equal(out.time.dt.year.max(), da.time.dt.year.max())
+        np.testing.assert_array_equal(out.time.max(), da.time.max())
+
+        with pytest.warns(None):
+            out = subset.subset_time(da, start_date=f"{yr_st}-07-15")
+        np.testing.assert_array_equal(out.time.dt.year.min(), int(yr_st))
+        np.testing.assert_array_equal(out.time.min().dt.month, 7)
+        np.testing.assert_array_equal(out.time.min().dt.day, 15)
+        np.testing.assert_array_equal(out.time.dt.year.max(), da.time.dt.year.max())
+        np.testing.assert_array_equal(out.time.max(), da.time.max())
+
+    def test_time_end_only(self):
+        da = xr.open_dataset(self.nc_poslons).tas
+        yr_ed = "2059"
+
+        # end date only
+        with pytest.warns(None):
+            out = subset.subset_time(da, end_date=f"{yr_ed}-01")
+        np.testing.assert_array_equal(out.time.dt.year.max(), int(yr_ed))
+        np.testing.assert_array_equal(out.time.max().dt.month, 1)
+        np.testing.assert_array_equal(out.time.max().dt.day, 31)
+        np.testing.assert_array_equal(out.time.min(), da.time.min())
+
+        with pytest.warns(None):
+            out = subset.subset_time(da, end_date=f"{yr_ed}-06-15")
+        np.testing.assert_array_equal(out.time.dt.year.max(), int(yr_ed))
+        np.testing.assert_array_equal(out.time.max().dt.month, 6)
+        np.testing.assert_array_equal(out.time.max().dt.day, 15)
+        np.testing.assert_array_equal(out.time.min(), da.time.min())
+
+    def test_time_incomplete_years(self):
+        da = xr.open_dataset(self.nc_poslons).tas
+        yr_st = "2050"
+        yr_ed = "2059"
+
+        out = subset.subset_time(
+            da, start_date=f"{yr_st}-07-01", end_date=f"{yr_ed}-06-30"
+        )
+        out1 = subset.subset_time(da, start_date=f"{yr_st}-07", end_date=f"{yr_ed}-06")
+        np.testing.assert_array_equal(out, out1)
+        np.testing.assert_array_equal(out.time.dt.year.min(), int(yr_st))
+        np.testing.assert_array_equal(out.time.min().dt.month, 7)
+        np.testing.assert_array_equal(out.time.min().dt.day, 1)
+        np.testing.assert_array_equal(out.time.dt.year.max(), int(yr_ed))
+        np.testing.assert_array_equal(out.time.max().dt.month, 6)
+        np.testing.assert_array_equal(out.time.max().dt.day, 30)
+
+
 class TestSubsetGridPoint:
     nc_poslons = os.path.join(
         TESTS_DATA, "cmip3", "tas.sresb1.giss_model_e_r.run1.atm.da.nc"
@@ -20,6 +142,23 @@ class TestSubsetGridPoint:
     )
     nc_2dlonlat = os.path.join(TESTS_DATA, "CRCM5", "tasmax_bby_198406_se.nc")
 
+    def test_time_simple(self):
+        da = xr.open_dataset(self.nc_poslons).tas
+        da = da.assign_coords(lon=(da.lon - 360))
+        lon = -72.4
+        lat = 46.1
+        yr_st = "2050"
+        yr_ed = "2059"
+
+        out = subset.subset_gridpoint(
+            da, lon=lon, lat=lat, start_date=yr_st, end_date=yr_ed
+        )
+        np.testing.assert_almost_equal(out.lon, lon, 1)
+        np.testing.assert_almost_equal(out.lat, lat, 1)
+        np.testing.assert_array_equal(len(np.unique(out.time.dt.year)), 10)
+        np.testing.assert_array_equal(out.time.dt.year.max(), int(yr_ed))
+        np.testing.assert_array_equal(out.time.dt.year.min(), int(yr_st))
+
     def test_dataset(self):
         da = xr.open_mfdataset(
             [self.nc_file, self.nc_file.replace("tasmax", "tasmin")],
@@ -32,147 +171,19 @@ class TestSubsetGridPoint:
         np.testing.assert_almost_equal(out.lat, lat, 1)
         np.testing.assert_array_equal(out.tasmin.shape, out.tasmax.shape)
 
-    def test_simple(self):
+    @pytest.mark.parametrize(
+        "lon,lat", [([-72.4], [46.1]), ([-67.4, -67.3], [43.1, 46.1])]
+    )
+    @pytest.mark.parametrize("add_distance", [True, False])
+    def test_simple(self, lat, lon, add_distance):
         da = xr.open_dataset(self.nc_file).tasmax
-        lon = -72.4
-        lat = 46.1
-        out = subset.subset_gridpoint(da, lon=lon, lat=lat)
+
+        out = subset.subset_gridpoint(da, lon=lon, lat=lat, add_distance=add_distance)
         np.testing.assert_almost_equal(out.lon, lon, 1)
         np.testing.assert_almost_equal(out.lat, lat, 1)
 
-        da = xr.open_dataset(self.nc_poslons).tas
-        da["lon"] -= 360
-        yr_st = 2050
-        yr_ed = 2059
-
-        out = subset.subset_gridpoint(
-            da, lon=lon, lat=lat, start_date=str(yr_st), end_date=str(yr_ed)
-        )
-        np.testing.assert_almost_equal(out.lon, lon, 1)
-        np.testing.assert_almost_equal(out.lat, lat, 1)
-        np.testing.assert_array_equal(len(np.unique(out.time.dt.year)), 10)
-        np.testing.assert_array_equal(out.time.dt.year.max(), yr_ed)
-        np.testing.assert_array_equal(out.time.dt.year.min(), yr_st)
-
-        # test time only
-        out = subset.subset_gridpoint(da, start_date=str(yr_st), end_date=str(yr_ed))
-        np.testing.assert_array_equal(len(np.unique(out.time.dt.year)), 10)
-        np.testing.assert_array_equal(out.time.dt.year.max(), yr_ed)
-        np.testing.assert_array_equal(out.time.dt.year.min(), yr_st)
-
-    def test_time_simple(self):
-        da = xr.open_dataset(self.nc_file).tasmax
-        lon = -72.4
-        lat = 46.1
-
-        da = xr.open_dataset(self.nc_poslons).tas
-        da["lon"] -= 360
-        yr_st = "2050"
-        yr_ed = "2059"
-
-        out = subset.subset_gridpoint(
-            da, lon=lon, lat=lat, start_date=yr_st, end_date=yr_ed
-        )
-        np.testing.assert_almost_equal(out.lon, lon, 1)
-        np.testing.assert_almost_equal(out.lat, lat, 1)
-        np.testing.assert_array_equal(len(np.unique(out.time.dt.year)), 10)
-        np.testing.assert_array_equal(out.time.dt.year.max(), int(yr_ed))
-        np.testing.assert_array_equal(out.time.dt.year.min(), int(yr_st))
-
-        # test different but equivalent strings
-        out = subset.subset_gridpoint(da, start_date=yr_st, end_date=yr_ed)
-        out1 = subset.subset_gridpoint(
-            da, start_date=f"{yr_st}-01", end_date=f"{yr_ed}-12"
-        )
-        out2 = subset.subset_gridpoint(
-            da, start_date=f"{yr_st}-01-01", end_date=f"{yr_ed}-12-31"
-        )
-        np.testing.assert_array_equal(out, out1)
-        np.testing.assert_array_equal(out, out2)
-        np.testing.assert_array_equal(len(np.unique(out.time.dt.year)), 10)
-        np.testing.assert_array_equal(out.time.dt.year.max(), int(yr_ed))
-        np.testing.assert_array_equal(out.time.dt.year.min(), int(yr_st))
-
-    def test_time_dates_outofbounds(self):
-        da = xr.open_dataset(self.nc_poslons).tas
-        da["lon"] -= 360
-        yr_st = "1950"
-        yr_ed = "2099"
-
-        with pytest.warns(UserWarning):
-            out = subset.subset_gridpoint(
-                da, start_date=f"{yr_st}-01", end_date=f"{yr_ed}-01"
-            )
-        np.testing.assert_array_equal(out.time.dt.year.min(), da.time.dt.year.min())
-        np.testing.assert_array_equal(out.time.dt.year.max(), da.time.dt.year.max())
-
-    def test_time_start_only(self):
-        da = xr.open_dataset(self.nc_poslons).tas
-        da["lon"] -= 360
-        yr_st = "2050"
-
-        # start date only
-        with pytest.warns(UserWarning):
-            out = subset.subset_gridpoint(da, start_date=f"{yr_st}-01")
-        np.testing.assert_array_equal(out.time.dt.year.min(), int(yr_st))
-        np.testing.assert_array_equal(out.time.dt.year.max(), da.time.dt.year.max())
-
-        with pytest.warns(UserWarning):
-            out = subset.subset_gridpoint(da, start_date=f"{yr_st}-07")
-        np.testing.assert_array_equal(out.time.dt.year.min(), int(yr_st))
-        np.testing.assert_array_equal(out.time.min().dt.month, 7)
-        np.testing.assert_array_equal(out.time.dt.year.max(), da.time.dt.year.max())
-        np.testing.assert_array_equal(out.time.max(), da.time.max())
-
-        with pytest.warns(UserWarning):
-            out = subset.subset_gridpoint(da, start_date=f"{yr_st}-07-15")
-        np.testing.assert_array_equal(out.time.dt.year.min(), int(yr_st))
-        np.testing.assert_array_equal(out.time.min().dt.month, 7)
-        np.testing.assert_array_equal(out.time.min().dt.day, 15)
-        np.testing.assert_array_equal(out.time.dt.year.max(), da.time.dt.year.max())
-        np.testing.assert_array_equal(out.time.max(), da.time.max())
-
-    def test_time_end_only(self):
-
-        da = xr.open_dataset(self.nc_poslons).tas
-        da["lon"] -= 360
-
-        yr_ed = "2059"
-
-        # end date only
-        with pytest.warns(UserWarning):
-            out = subset.subset_gridpoint(da, end_date=f"{yr_ed}-01")
-        np.testing.assert_array_equal(out.time.dt.year.max(), int(yr_ed))
-        np.testing.assert_array_equal(out.time.max().dt.month, 1)
-        np.testing.assert_array_equal(out.time.max().dt.day, 31)
-        np.testing.assert_array_equal(out.time.min(), da.time.min())
-
-        with pytest.warns(UserWarning):
-            out = subset.subset_gridpoint(da, end_date=f"{yr_ed}-06-15")
-        np.testing.assert_array_equal(out.time.dt.year.max(), int(yr_ed))
-        np.testing.assert_array_equal(out.time.max().dt.month, 6)
-        np.testing.assert_array_equal(out.time.max().dt.day, 15)
-        np.testing.assert_array_equal(out.time.min(), da.time.min())
-
-    def test_time_incomplete_years(self):
-        da = xr.open_dataset(self.nc_poslons).tas
-        da["lon"] -= 360
-        yr_st = "2050"
-        yr_ed = "2059"
-
-        out = subset.subset_gridpoint(
-            da, start_date=f"{yr_st}-07-01", end_date=f"{yr_ed}-06-30"
-        )
-        out1 = subset.subset_gridpoint(
-            da, start_date=f"{yr_st}-07", end_date=f"{yr_ed}-06"
-        )
-        np.testing.assert_array_equal(out, out1)
-        np.testing.assert_array_equal(out.time.dt.year.min(), int(yr_st))
-        np.testing.assert_array_equal(out.time.min().dt.month, 7)
-        np.testing.assert_array_equal(out.time.min().dt.day, 1)
-        np.testing.assert_array_equal(out.time.dt.year.max(), int(yr_ed))
-        np.testing.assert_array_equal(out.time.max().dt.month, 6)
-        np.testing.assert_array_equal(out.time.max().dt.day, 30)
+        assert ("site" in out.dims) ^ (len(lat) == 1)
+        assert ("distance" in out.coords) ^ (not add_distance)
 
     def test_irregular(self):
 
@@ -182,6 +193,14 @@ class TestSubsetGridPoint:
         out = subset.subset_gridpoint(da, lon=lon, lat=lat)
         np.testing.assert_almost_equal(out.lon, lon, 1)
         np.testing.assert_almost_equal(out.lat, lat, 1)
+        assert "site" not in out.dims
+
+        lon = [-72.4, -67.1]
+        lat = [46.1, 48.2]
+        out = subset.subset_gridpoint(da, lon=lon, lat=lat)
+        np.testing.assert_almost_equal(out.lon, lon, 1)
+        np.testing.assert_almost_equal(out.lat, lat, 1)
+        assert "site" in out.dims
 
         # dask for lon lat
         da.lon.chunk({"rlon": 10})
@@ -271,8 +290,8 @@ class TestSubsetGridPoint:
         da = xr.open_dataset(self.nc_poslons).tas
         lon = -72.5
         lat = 46.2
-        with pytest.raises(ValueError):
-            subset.subset_gridpoint(da, lon=lon, lat=lat, tolerance=1)
+        out = subset.subset_gridpoint(da, lon=lon, lat=lat, tolerance=1)
+        assert out.isnull().all()
 
         subset.subset_gridpoint(da, lon=lon, lat=lat, tolerance=1e5)
 
@@ -314,7 +333,7 @@ class TestSubsetBbox:
         assert np.all(out.lat <= np.max(self.lat))
 
         da = xr.open_dataset(self.nc_poslons).tas
-        da["lon"] -= 360
+        da = da.assign_coords(lon=(da.lon - 360))
         yr_st = 2050
         yr_ed = 2059
 
@@ -334,10 +353,10 @@ class TestSubsetBbox:
         np.testing.assert_array_equal(out.time.dt.year.max(), yr_ed)
         np.testing.assert_array_equal(out.time.dt.year.min(), yr_st)
 
-        with pytest.warns(Warning):
-            out = subset.subset_bbox(
-                da, lon_bnds=self.lon, lat_bnds=self.lat, start_date=str(yr_st)
-            )
+        out = subset.subset_bbox(
+            da, lon_bnds=self.lon, lat_bnds=self.lat, start_date=str(yr_st)
+        )
+
         assert out.lon.values.size != 0
         assert out.lat.values.size != 0
         assert np.all(out.lon >= np.min(self.lon))
@@ -347,10 +366,10 @@ class TestSubsetBbox:
         np.testing.assert_array_equal(out.time.dt.year.max(), da.time.dt.year.max())
         np.testing.assert_array_equal(out.time.dt.year.min(), yr_st)
 
-        with pytest.warns(Warning):
-            out = subset.subset_bbox(
-                da, lon_bnds=self.lon, lat_bnds=self.lat, end_date=str(yr_ed)
-            )
+        out = subset.subset_bbox(
+            da, lon_bnds=self.lon, lat_bnds=self.lat, end_date=str(yr_ed)
+        )
+
         assert out.lon.values.size != 0
         assert out.lat.values.size != 0
         assert np.all(out.lon >= np.min(self.lon))
@@ -486,7 +505,7 @@ class TestSubsetBbox:
 
     def test_time(self):
         da = xr.open_dataset(self.nc_poslons).tas
-        da["lon"] -= 360
+        da = da.assign_coords(lon=(da.lon - 360))
 
         out = subset.subset_bbox(
             da,
@@ -545,9 +564,9 @@ class TestSubsetBbox:
 
     def test_warnings(self):
         da = xr.open_dataset(self.nc_poslons).tas
-        da["lon"] -= 360
+        da = da.assign_coords(lon=(da.lon - 360))
 
-        with pytest.warns(FutureWarning):
+        with pytest.raises(TypeError):
             subset.subset_bbox(
                 da, lon_bnds=self.lon, lat_bnds=self.lat, start_yr=2050, end_yr=2059
             )
@@ -562,7 +581,7 @@ class TestSubsetBbox:
         assert (
             '"start_yr" and "end_yr" (type: int) are being deprecated. Temporal subsets will soon exclusively'
             ' support "start_date" and "end_date" (type: str) using formats of "%Y", "%Y-%m" or "%Y-%m-%d".'
-            not in [q.message for q in record]
+            not in [str(q.message) for q in record]
         )
 
 
@@ -608,7 +627,8 @@ class TestSubsetShape:
                     sub[vari].sel(lon=lon1, lat=lat1), ds[vari].sel(lon=lon1, lat=lat1)
                 )
 
-    def test_wraps(self):
+    @pytest.mark.parametrize("vectorize", [True, False])
+    def test_wraps(self, tmp_netcdf_filename, vectorize):
         ds = xr.open_dataset(self.nc_file)
 
         # Polygon crosses meridian, a warning should be raised
@@ -617,6 +637,7 @@ class TestSubsetShape:
 
         # No time subsetting should occur.
         assert len(sub.tas) == 12
+
         # Average temperature at surface for region in January (time=0)
         np.testing.assert_array_almost_equal(
             float(np.mean(sub.tas.isel(time=0))), 285.064453
@@ -624,21 +645,32 @@ class TestSubsetShape:
         self.compare_vals(ds, sub, "tas")
 
         poly = gpd.read_file(self.meridian_multi_geojson)
-        subtas = subset.subset_shape(ds.tas, poly)
+        subtas = subset.subset_shape(ds.tas, poly, vectorize=vectorize)
         np.testing.assert_array_almost_equal(
             float(np.mean(subtas.isel(time=0))), 281.091553
         )
 
-    def test_no_wraps(self):
+        assert sub.crs.prime_meridian_name == "Greenwich"
+        assert sub.crs.grid_mapping_name == "latitude_longitude"
+
+        sub.to_netcdf(tmp_netcdf_filename)
+        assert tmp_netcdf_filename.exists()
+        with xr.open_dataset(filename_or_obj=tmp_netcdf_filename) as f:
+            assert {"tas", "crs"}.issubset(set(f.data_vars))
+            subset.subset_shape(ds, self.meridian_multi_geojson, vectorize=vectorize)
+
+    @pytest.mark.parametrize("vectorize", [True, False])
+    def test_no_wraps(self, tmp_netcdf_filename, vectorize):
         ds = xr.open_dataset(self.nc_file)
 
         with pytest.warns(None) as record:
-            sub = subset.subset_shape(ds, self.poslons_geojson)
+            sub = subset.subset_shape(ds, self.poslons_geojson, vectorize=vectorize)
 
         self.compare_vals(ds, sub, "tas")
 
         # No time subsetting should occur.
         assert len(sub.tas) == 12
+
         # Average temperature at surface for region in January (time=0)
         np.testing.assert_array_almost_equal(
             float(np.mean(sub.tas.isel(time=0))), 276.732483
@@ -647,9 +679,18 @@ class TestSubsetShape:
         assert (
             '"Geometry crosses the Greenwich Meridian. Proceeding to split polygon at Greenwich."'
             '" This feature is experimental. Output might not be accurate."'
-            not in [q.message for q in record]
+            not in [str(q.message) for q in record]
         )
 
+        assert sub.crs.prime_meridian_name == "Greenwich"
+        assert sub.crs.grid_mapping_name == "latitude_longitude"
+
+        sub.to_netcdf(tmp_netcdf_filename)
+        assert tmp_netcdf_filename.exists()
+        with xr.open_dataset(filename_or_obj=tmp_netcdf_filename) as f:
+            assert {"tas", "crs"}.issubset(set(f.data_vars))
+            subset.subset_shape(ds, self.poslons_geojson, vectorize=vectorize)
+
     def test_all_neglons(self):
         ds = xr.open_dataset(self.nc_file_neglons)
 
@@ -692,10 +733,10 @@ class TestSubsetShape:
         assert (
             '"Geometry crosses the Greenwich Meridian. Proceeding to split polygon at Greenwich."'
             '" This feature is experimental. Output might not be accurate."'
-            not in [q.message for q in record]
+            not in [str(q.message) for q in record]
         )
 
-    def test_small_poly_buffer(self):
+    def test_small_poly_buffer(self, tmp_netcdf_filename):
         ds = xr.open_dataset(self.nc_file)
 
         with pytest.raises(ValueError):
@@ -709,6 +750,14 @@ class TestSubsetShape:
         assert len(sub.lon.values) == 3
         assert len(sub.lat.values) == 3
 
+        assert sub.crs.prime_meridian_name == "Greenwich"
+        assert sub.crs.grid_mapping_name == "latitude_longitude"
+
+        sub.to_netcdf(tmp_netcdf_filename)
+        assert tmp_netcdf_filename.exists()
+        with xr.open_dataset(filename_or_obj=tmp_netcdf_filename) as f:
+            assert {"tas", "crs"}.issubset(set(f.data_vars))
+
     def test_mask_multiregions(self):
         ds = xr.open_dataset(self.nc_file)
         regions = gpd.read_file(self.multi_regions_geojson)
@@ -744,7 +793,7 @@ class TestDistance:
             np.random.rand(lon.size, lat.size), dims=["lon", "lat"]
         )
 
-        d = subset.distance(da, -34, 56)
+        d = subset.distance(da, lon=-34, lat=56).squeeze("site")
         assert d.dims == da.data.dims
         assert d.shape == da.data.shape
         assert d.units == "m"
