@@ -16,7 +16,6 @@ __all__ = [
     "heat_wave_index",
     "hot_spell_frequency",
     "hot_spell_max_length",
-    "tg",
     "tg_mean",
     "tg10p",
     "tg90p",
@@ -209,7 +208,7 @@ hot_spell_frequency = Tasmax(
 )
 
 hot_spell_max_length = Tasmax(
-    identifier="heat_wave_max_length",
+    identifier="hot_spell_max_length",
     units="days",
     standard_name="spell_length_of_days_with_air_temperature_above_threshold",
     long_name="Maximum length of hot spell events (Tmax > {thresh_tasmax} for >= {window} days)",
@@ -221,16 +220,6 @@ hot_spell_max_length = Tasmax(
     compute=indices.hot_spell_max_length,
 )
 
-tg = TasminTasmax(
-    identifier="tg",
-    units="K",
-    standard_name="air_temperature",
-    long_name="Daily mean temperature",
-    description="Estimated mean temperature from maximum and minimum temperatures",
-    cell_methods="",
-    compute=indices.tas,
-)
-
 tg_mean = Tas(
     identifier="tg_mean",
     units="K",
